""" Views for the Curate Data Structure REST API
"""

import logging

from django.http import Http404
from drf_spectacular.types import OpenApiTypes
from drf_spectacular.utils import (
    OpenApiParameter,
    OpenApiExample,
    extend_schema,
    OpenApiResponse,
)
from rest_framework import status
from rest_framework.exceptions import ValidationError
from rest_framework.permissions import IsAdminUser, IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView

from core_curate_app.components.curate_data_structure import (
    api as data_structure_api,
)
from core_curate_app.components.curate_data_structure.models import (
    CurateDataStructure,
)
from core_curate_app.rest.curate_data_structure.admin_serializers import (
    CurateDataStructureAdminSerializer,
)
from core_curate_app.rest.curate_data_structure.serializers import (
    CurateDataStructureSerializer,
)
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.commons import exceptions

logger = logging.getLogger(__name__)


@extend_schema(
    tags=["Curate Data Structure"],
    description="List all Curate Data Structure, or create a new one",
)
class AdminCurateDataStructureList(APIView):
    """List all Curate Data Structure, or create a new one."""

    permission_classes = (IsAdminUser,)
    serializer = CurateDataStructureAdminSerializer

    @extend_schema(
        summary="Get all Curate Data Structure",
        description="Retrieve a list of all Curate Data Structures",
        responses={
            200: CurateDataStructureAdminSerializer(many=True),
            403: OpenApiResponse(description="Access Forbidden"),
            500: OpenApiResponse(description="Internal server error"),
        },
    )
    def get(self, request):
        """Get all user Curate Data Structure
        Args:
            request: HTTP request
        Returns:
            - code: 200
              content: List of curate data structure
            - code: 403
              content: Forbidden
            - code: 500
              content: Internal server error
        """
        if not request.user.is_superuser:
            return Response(status=status.HTTP_403_FORBIDDEN)
        try:
            # Get object
            object_list = CurateDataStructure.get_all()
            # Serialize object
            serializer = self.serializer(object_list, many=True)
            # Return response
            return Response(serializer.data, status=status.HTTP_200_OK)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )

    @extend_schema(
        summary="Create a Curate Data Structure",
        description="Create a new Curate Data Structure",
        request=CurateDataStructureAdminSerializer,
        responses={
            201: CurateDataStructureAdminSerializer,
            400: OpenApiResponse(description="Validation error"),
            404: OpenApiResponse(description="Template was not found"),
            500: OpenApiResponse(description="Internal server error"),
        },
        examples=[
            OpenApiExample(
                "Create a Curate Data Structure",
                summary="Create a new Curate Data Structure",
                description="Create a new Curate Data Structure with the provided parameters",
                request_only=True,
                value={
                    "user": "1",
                    "name": "name",
                    "form_string": "<xml></xml>",
                    "template": "5eb1cc6d53d26cbd4085c722",
                    "data_structure_element_root": "5eb36cc1b72a6298744d746a",
                    "data": "5eb36ca0b72a6298744d724b",
                },
            ),
        ],
    )
    def post(self, request):
        """Create a Curate Data Structure
        Parameters:
            {
              "user": "1",
              "name": "name",
              "form_string": "<xml></xml>",
              "template": "123",
              "data_structure_element_root": "123",
              "data": "123"
            }
        Args:
            request: HTTP request
        Returns:
            - code: 201
              content: Created data
            - code: 400
              content: Validation error
            - code: 404
              content: Template was not found
            - code: 500
              content: Internal server error
        """
        if not request.user.is_superuser:
            return Response(status=status.HTTP_403_FORBIDDEN)
        try:
            # Build serializer
            serializer = self.serializer(
                data=request.data, context={"request": request}
            )
            # Validate data
            serializer.is_valid(raise_exception=True)
            # Save data
            serializer.save()
            # Return the serialized data
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        except AccessControlError as access_control_exception:
            content = {"message": str(access_control_exception)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except ValidationError as validation_exception:
            content = {"message": validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except exceptions.DoesNotExist:
            content = {"message": "Object not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )


@extend_schema(
    tags=["Curate Data Structure"],
    description="List Curate Data Structure by user, create a new one",
)
class CurateDataStructureList(APIView):
    """List Curate Data Structure by user, create a new one."""

    permission_classes = (IsAuthenticated,)
    serializer = CurateDataStructureSerializer

    @extend_schema(
        summary="Get all Curate Data Structure by user",
        description="Retrieve a list of all Curate Data Structures for the current user",
        responses={
            200: CurateDataStructureSerializer(many=True),
            403: OpenApiResponse(description="Access Forbidden"),
            500: OpenApiResponse(description="Internal server error"),
        },
    )
    def get(self, request):
        """Get all user Curate Data Structure
        Args:
            request: HTTP request
        Returns:
            - code: 200
              content: List of curate data structure
            - code: 403
              content: Forbidden
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            object_list = data_structure_api.get_all_by_user(request.user)
            # Serialize object
            serializer = self.serializer(object_list, many=True)
            # Return response
            return Response(serializer.data, status=status.HTTP_200_OK)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )

    @extend_schema(
        summary="Create a Curate Data Structure",
        description="Create a new Curate Data Structure",
        request=CurateDataStructureSerializer,
        responses={
            201: CurateDataStructureSerializer,
            400: OpenApiResponse(description="Validation error"),
            404: OpenApiResponse(description="Template was not found"),
            500: OpenApiResponse(description="Internal server error"),
        },
        examples=[
            OpenApiExample(
                "Create a Curate Data Structure",
                summary="Create a new Curate Data Structure",
                description="Create a new Curate Data Structure with the provided parameters",
                request_only=True,
                value={
                    "name": "name",
                    "form_string": "<xml></xml>",
                    "template": "123",
                    "data_structure_element_root": "123",
                    "data": "123",
                },
            ),
        ],
    )
    def post(self, request):
        """Create a Curate Data Structure
        Parameters:
            {
              "name": "name",
              "form_string": "<xml></xml>",
              "template": "5eb1cc6d53d26cbd4085c722",
              "data_structure_element_root": "5eb36cc1b72a6298744d746a",
              "data": "5eb36ca0b72a6298744d724b"
            }
        Args:
            request: HTTP request
        Returns:
            - code: 201
              content: Created data
            - code: 400
              content: Validation error
            - code: 404
              content: Template was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Build serializer
            serializer = self.serializer(
                data=request.data, context={"request": request}
            )
            # Validate data
            serializer.is_valid(raise_exception=True)
            # Save data
            serializer.save()
            # Return the serialized data
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        except ValidationError as validation_exception:
            content = {"message": validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except exceptions.DoesNotExist:
            content = {"message": "Object not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )


@extend_schema(
    tags=["Curate Data Structure"],
    description="Retrieve, update or delete a Curate Data Structure",
)
class CurateDataStructureDetail(APIView):
    """Retrieve, update or delete a Curate Data Structure"""

    permission_classes = (IsAuthenticated,)
    serializer = CurateDataStructureSerializer

    def get_object(self, request, pk):
        """Get Curate Data Structure from db
        Args:
            request: HTTP request
            pk: ObjectId
        Returns:
            Data
        """
        try:
            return data_structure_api.get_by_id(pk, request.user)
        except exceptions.DoesNotExist:
            raise Http404

    @extend_schema(
        summary="Retrieve a Curate Data Structure",
        description="Retrieve a Curate Data Structure by ID",
        parameters=[
            OpenApiParameter(
                name="id",
                type=OpenApiTypes.STR,
                location=OpenApiParameter.PATH,
                description="Curate Data Structure ID",
            ),
        ],
        responses={
            200: CurateDataStructureSerializer,
            403: OpenApiResponse(description="Access Forbidden"),
            404: OpenApiResponse(description="Object was not found"),
            500: OpenApiResponse(description="Internal server error"),
        },
    )
    def get(self, request, pk):
        """Retrieve a Curate Data Structure
        Args:
            request: HTTP request
            pk: ObjectId
        Returns:
            - code: 200
              content: Curate Data Structure
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            data_structure_object = self.get_object(request, pk)
            # Serialize object
            serializer = self.serializer(data_structure_object)
            # Return response
            return Response(serializer.data)
        except AccessControlError as access_control_exception:
            content = {"message": str(access_control_exception)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Http404:
            content = {"message": "Data structure not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )

    @extend_schema(
        summary="Delete a Curate Data Structure",
        description="Delete a Curate Data Structure by ID",
        parameters=[
            OpenApiParameter(
                name="id",
                type=OpenApiTypes.STR,
                location=OpenApiParameter.PATH,
                description="Curate Data Structure ID",
            ),
        ],
        responses={
            204: None,
            403: OpenApiResponse(description="Access Forbidden"),
            404: OpenApiResponse(description="Object was not found"),
            500: OpenApiResponse(description="Internal server error"),
        },
    )
    def delete(self, request, pk):
        """Delete a Curate Data Structure
        Args:
            request: HTTP request
            pk: ObjectId
        Returns:
            - code: 204
              content: Deletion succeed
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            data_structure_object = self.get_object(request, pk)
            # delete object
            data_structure_api.delete(data_structure_object, request.user)
            # Return response
            return Response(status=status.HTTP_204_NO_CONTENT)
        except AccessControlError as access_control_exception:
            content = {"message": str(access_control_exception)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Http404:
            content = {"message": "Data structure not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )

    @extend_schema(
        summary="Update a Curate Data Structure",
        description="Update a Curate Data Structure by ID",
        parameters=[
            OpenApiParameter(
                name="id",
                type=OpenApiTypes.STR,
                location=OpenApiParameter.PATH,
                description="Curate Data Structure ID",
            ),
        ],
        request=CurateDataStructureSerializer(partial=True),
        responses={
            200: CurateDataStructureSerializer,
            400: OpenApiResponse(description="Validation error"),
            403: OpenApiResponse(description="Access Forbidden"),
            404: OpenApiResponse(description="Object was not found"),
            500: OpenApiResponse(description="Internal server error"),
        },
        examples=[
            OpenApiExample(
                "Update a Curate Data Structure",
                summary="Update a Curate Data Structure",
                description="Update a Curate Data Structure with the provided parameters",
                request_only=True,
                value={
                    "name": "new_name",
                    "form_string": "<new_xml></new_xml>",
                    "data": "[data_id]",
                    "data_structure_element_root": "[data_structure_element_root_id]",
                },
            ),
        ],
    )
    def patch(self, request, pk):
        """Update a Curate Data Structure
        Parameters:
            {
              "name": "new_name",
              "form_string": "<new_xml></new_xml>",
              "data": "[data_id]",
              "data_structure_element_root": "[data_structure_element_root_id]"
            }
        Args:
            request: HTTP request
            pk: ObjectId
        Returns:
            - code: 200
              content: Updated data structure
            - code: 400
              content: Validation error
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            data_structure_object = self.get_object(request, pk)
            # Build serializer
            serializer = self.serializer(
                instance=data_structure_object,
                data=request.data,
                partial=True,
                context={"request": request},
            )
            # Validate data
            serializer.is_valid(raise_exception=True)
            # Save data
            serializer.save()
            return Response(serializer.data, status=status.HTTP_200_OK)
        except AccessControlError as access_control_exception:
            content = {"message": str(access_control_exception)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except ValidationError as validation_exception:
            content = {"message": validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Http404:
            content = {"message": "Data structure not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )



""" Unit tests for views from `views.admin.ajax`.
"""

from unittest.case import TestCase
from unittest.mock import patch

from django.http import (
    HttpResponse,
    HttpResponseBadRequest,
)
from django.test import RequestFactory

from core_curate_app.views.admin.ajax import delete_record_drafts
from core_main_app.components.data.models import Data
from core_main_app.utils.tests_tools.MockUser import create_mock_user


class TestDeleteDataDrafts(TestCase):
    """Test Delete Data Drafts"""

    def setUp(self):
        """setUp
        Returns:
        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(
            user_id="1", is_staff=True, is_superuser=True
        )

    @patch(
        "core_curate_app.components.curate_data_structure.api.delete_curate_data_structures_by_data"
    )
    @patch("core_main_app.components.data.api.get_by_id")
    def test_delete_data_drafts_returns_http_response(
        self,
        mock_data_get_by_id,
        mock_delete_data_structures,
    ):
        """test_delete_data_drafts_returns_http_response


        Returns:


        """
        mock_data_get_by_id.return_value = Data()
        mock_delete_data_structures.return_value = []
        request = self.factory.delete(
            "core-admin:core_curate_app_delete_data_drafts"
        )

        request.user = self.user1
        response = delete_record_drafts(request, 1)

        self.assertTrue(isinstance(response, HttpResponse))

    def test_delete_data_drafts_returns_error_when_data_not_found(self):
        """test_delete_data_drafts_returns_error_when_data_not_found


        Returns:


        """
        request = self.factory.delete(
            "core-admin:core_curate_app_delete_data_drafts"
        )

        request.user = self.user1
        response = delete_record_drafts(request, 1)

        self.assertTrue(isinstance(response, HttpResponseBadRequest))

    @patch(
        "core_curate_app.components.curate_data_structure.api.delete_curate_data_structures_by_data"
    )
    @patch("core_main_app.components.data.api.get_by_id")
    def test_delete_data_drafts_returns_error(
        self, mock_data_get_by_id, mock_delete_data_structures
    ):
        """test_delete_data_drafts_returns_error


        Returns:


        """
        mock_data_get_by_id.return_value = Data()
        mock_delete_data_structures.side_effect = Exception()
        request = self.factory.delete(
            "core-admin:core_curate_app_delete_data_drafts"
        )

        request.user = self.user1
        response = delete_record_drafts(request, 1)

        self.assertTrue(isinstance(response, HttpResponseBadRequest))



""" Test access to views from `views.user.ajax`.
"""

from django.core.files.uploadedfile import SimpleUploadedFile
from django.test import RequestFactory
from tests.components.curate_data_structure.fixtures.fixtures import (
    DataStructureFixtures,
)

from core_curate_app.views.user import ajax as curate_ajax
from core_main_app.utils.integration_tests.integration_base_test_case import (
    IntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user

fixture_data_structure = DataStructureFixtures()


class TestStartCuratePost(IntegrationBaseTestCase):
    """Test Start Curate Post"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.user2 = create_mock_user(user_id="3")
        self.fixture = DataStructureFixtures()
        self.fixture.insert_data()

    def test_xsd_template_start_curate_new_returns_http_200(self):
        """test_xsd_template_start_curate_new_returns_http_200

        Returns:

        """
        data = {
            "curate_form": "new",
            "document_name": "test",
            "template_format": "XSD",
            "forms": "",
            "template_id": str(self.fixture.data_structure_1.template.id),
        }
        request = self.factory.post("core_curate_start", data)
        request.user = self.user1
        response = curate_ajax._start_curate_post(request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue("/enter-data/" in response.content.decode("utf-8"))

    def test_xsd_template_start_curate_new_in_text_editor_returns_http_200(
        self,
    ):
        """test_xsd_template_start_curate_new_in_text_editor_returns_http_200

        Returns:

        """
        data = {
            "curate_form": "new",
            "document_name": "test",
            "forms": "",
            "template_format": "XSD",
            "text_editor": "on",
            "template_id": str(self.fixture.data_structure_1.template.id),
        }
        request = self.factory.post("core_curate_start", data)
        request.user = self.user1
        response = curate_ajax._start_curate_post(request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(
            ("/xml-editor/form" in response.content.decode("utf-8"))
        )

    def test_xsd_template_start_curate_upload_returns_http_200(self):
        """test_xsd_template_start_curate_upload_returns_http_200

        Returns:

        """
        data = {
            "curate_form": "upload",
            "document_name": "",
            "template_format": "XSD",
            "forms": "",
            "template_id": str(self.fixture.data_structure_1.template.id),
        }
        content = "<tag></tag>"
        files = SimpleUploadedFile("test.xml", content.encode("utf-8"))

        request = self.factory.post("core_curate_start", data)
        request.FILES["file"] = files
        request.user = self.user1
        response = curate_ajax._start_curate_post(request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(("/enter-data/" in response.content.decode("utf-8")))

    def test_xsd_template_start_curate_upload_bad_content_format_returns_error_400(
        self,
    ):
        """test_xsd_template_start_curate_upload_bad_content_format_returns_error_400

        Returns:

        """
        data = {
            "curate_form": "upload",
            "document_name": "",
            "template_format": "XSD",
            "forms": "",
            "template_id": str(self.fixture.data_structure_1.template.id),
        }
        content = "bad format"
        files = SimpleUploadedFile("test.xml", content.encode("utf-8"))

        request = self.factory.post("core_curate_start", data)
        request.FILES["file"] = files
        request.user = self.user1
        response = curate_ajax._start_curate_post(request)
        self.assertEqual(response.status_code, 400)
        self.assertTrue(
            (
                "the file is not well formed XSD"
                in response.content.decode("utf-8")
            )
        )

    def test_xsd_template_start_curate_in_text_editor_upload_returns_http_200(
        self,
    ):
        """test_xsd_template_start_curate_in_text_editor_upload_returns_http_200

        Returns:

        """
        data = {
            "curate_form": "upload",
            "document_name": "",
            "template_format": "XSD",
            "forms": "",
            "text_editor": "on",
            "template_id": str(self.fixture.data_structure_1.template.id),
        }
        content = "<tag></tag>"
        files = SimpleUploadedFile("test.xml", content.encode("utf-8"))

        request = self.factory.post("core_curate_start", data)
        request.FILES["file"] = files
        request.user = self.user1
        response = curate_ajax._start_curate_post(request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(
            ("/xml-editor/form" in response.content.decode("utf-8"))
        )

    def test_xsd_template_start_curate_open_form_returns_http_200(self):
        """test_xsd_template_start_curate_open_form_returns_http_200

        Returns:

        """
        data = {
            "curate_form": "open",
            "document_name": "test",
            "template_format": "XSD",
            "forms": str(self.fixture.data_structure_1.id),
            "template_id": str(self.fixture.data_structure_1.template.id),
        }
        request = self.factory.post("core_curate_start", data)
        request.user = self.user1
        response = curate_ajax._start_curate_post(request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(("/enter-data/" in response.content.decode("utf-8")))

    def test_xsd_template_start_curate_open_form_in_text_editor_returns_http_200(
        self,
    ):
        """test_xsd_template_start_curate_open_form_in_text_editor_returns_http_200

        Returns:

        """
        data = {
            "curate_form": "open",
            "document_name": "test",
            "template_format": "XSD",
            "forms": str(self.fixture.data_structure_1.id),
            "text_editor": "on",
            "template_id": str(self.fixture.data_structure_1.template.id),
        }
        request = self.factory.post("core_curate_start", data)
        request.user = self.user1
        response = curate_ajax._start_curate_post(request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(
            ("/xml-editor/form" in response.content.decode("utf-8"))
        )

    def test_json_template_start_curate_new_returns_http_200(self):
        """test_json_template_start_curate_new_returns_http_200

        Returns:

        """
        data = {
            "curate_form": "new",
            "document_name": "test_json",
            "forms": "",
            "template_format": "JSON",
            "template_id": str(self.fixture.template_json.id),
        }
        request = self.factory.post("core_curate_start", data)
        request.user = self.user2
        response = curate_ajax._start_curate_post(request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue("/enter-data/" in response.content.decode("utf-8"))

    def test_json_template_start_curate_new_in_text_editor_returns_http_200(
        self,
    ):
        """test_json_template_start_curate_new_in_text_editor_returns_http_200

        Returns:

        """
        data = {
            "curate_form": "new",
            "document_name": "test_json",
            "forms": "",
            "template_format": "JSON",
            "text_editor": "on",
            "template_id": str(self.fixture.template_json.id),
        }
        request = self.factory.post("core_curate_start", data)
        request.user = self.user2
        response = curate_ajax._start_curate_post(request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(
            ("/json-editor/form" in response.content.decode("utf-8"))
        )

    def test_json_template_start_curate_upload_returns_http_200(self):
        """test_json_template_start_curate_upload_returns_http_200

        Returns:

        """
        data = {
            "curate_form": "upload",
            "document_name": "",
            "template_format": "JSON",
            "forms": "",
            "text_editor": "on",
            "template_id": str(self.fixture.template_json.id),
        }
        content = "{}"
        files = SimpleUploadedFile("test_json.json", content.encode("utf-8"))

        request = self.factory.post("core_curate_start", data)
        request.FILES["file"] = files
        request.user = self.user2
        response = curate_ajax._start_curate_post(request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(
            ("/json-editor/form" in response.content.decode("utf-8"))
        )

    def test_json_template_start_curate_upload_bad_content_format_returns_error_400(
        self,
    ):
        """test_json_template_start_curate_upload_bad_content_format_returns_error_400

        Returns:

        """
        data = {
            "curate_form": "upload",
            "document_name": "",
            "template_format": "JSON",
            "forms": "",
            "template_id": str(self.fixture.template_json.id),
        }
        content = "{ bad format"
        files = SimpleUploadedFile("test_json.json", content.encode("utf-8"))

        request = self.factory.post("core_curate_start", data)
        request.FILES["file"] = files
        request.user = self.user2
        response = curate_ajax._start_curate_post(request)
        self.assertEqual(response.status_code, 400)
        self.assertTrue(
            (
                "the file is not well formed JSON"
                in response.content.decode("utf-8")
            )
        )

    def test_json_template_start_curate_in_text_editor_upload_returns_http_200(
        self,
    ):
        """test_json_template_start_curate_in_text_editor_upload_returns_http_200

        Returns:

        """
        data = {
            "curate_form": "upload",
            "document_name": "",
            "template_format": "JSON",
            "forms": "",
            "text_editor": "on",
            "template_id": str(self.fixture.template_json.id),
        }
        content = "{}"
        files = SimpleUploadedFile("test_json.json", content.encode("utf-8"))

        request = self.factory.post("core_curate_start", data)
        request.FILES["file"] = files
        request.user = self.user2
        response = curate_ajax._start_curate_post(request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(
            ("/json-editor/form" in response.content.decode("utf-8"))
        )

    def test_json_template_start_curate_open_form_returns_http_200(self):
        """test_json_template_start_curate_open_form_returns_http_200

        Returns:

        """
        data = {
            "curate_form": "open",
            "document_name": "test_json",
            "template_format": "JSON",
            "text_editor": "on",
            "forms": str(self.fixture.data_structure_json_2.id),
            "template_id": str(self.fixture.template_json.id),
        }
        request = self.factory.post("core_curate_start", data)
        request.user = self.user2
        response = curate_ajax._start_curate_post(request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(
            ("/json-editor/form" in response.content.decode("utf-8"))
        )

    def test_start_curate_upload_bad_format_returns_error_400(self):
        """test_start_curate_upload_bad_format_returns_error_400

        Returns:

        """
        data = {
            "curate_form": "upload",
            "document_name": "",
            "template_format": "BAD_FORMAT",
            "forms": "",
            "template_id": str(self.fixture.template_unsupported_format.id),
        }
        content = "test"
        files = SimpleUploadedFile("test.bad_format", content.encode("utf-8"))

        request = self.factory.post("core_curate_start", data)
        request.FILES["file"] = files
        request.user = self.user2
        response = curate_ajax._start_curate_post(request)
        self.assertEqual(response.status_code, 400)
        self.assertTrue(
            "Template format not supported."
            in response.content.decode("utf-8")
        )



""" Test forms from `views.user.forms`.
"""

from unittest.case import TestCase
from unittest.mock import MagicMock

from django.core.exceptions import ValidationError
from django.test import override_settings

from core_curate_app.views.user.forms import (
    _file_size_validator,
    OpenForm,
    HiddenFieldsForm,
)
from core_main_app.settings import MAX_DOCUMENT_EDITING_SIZE


class TestFileSizeValidator(TestCase):
    """Test _file_size_validator"""

    def test_file_size_validator_raises_error_when_file_too_large(
        self,
    ):
        """test_file_size_validator_raises_error_when_file_too_large

        Returns:

        """
        mock_field = MagicMock()
        mock_field.size = MAX_DOCUMENT_EDITING_SIZE + 1
        with self.assertRaises(ValidationError):
            _file_size_validator(mock_field)

    def test_file_size_validator_raises_ok_when_file_size_under_limit(
        self,
    ):
        """test_file_size_validator_raises_ok_when_file_size_under_limit

        Returns:

        """
        mock_field = MagicMock()
        mock_field.size = MAX_DOCUMENT_EDITING_SIZE - 1
        _file_size_validator(mock_field)


class TestOpenForm(TestCase):
    """Test Open Form"""

    @override_settings(BOOTSTRAP_VERSION="4.6.2")
    def test_open_form_bootstrap_v4(self):
        """test_open_form_bootstrap_v4

        Returns:

        """
        data = {"forms": "test"}
        form = OpenForm(data)
        self.assertEqual(
            form.fields["forms"].widget.attrs["class"], "form-control"
        )

    @override_settings(BOOTSTRAP_VERSION="5.1.3")
    def test_open_form_bootstrap_v5(self):
        """test_open_form_bootstrap_v5

        Returns:

        """
        data = {"forms": "test"}
        form = OpenForm(data)
        self.assertEqual(
            form.fields["forms"].widget.attrs["class"], "form-select"
        )


class TestHiddenFieldsForm(TestCase):
    """Test Hidden Fields Form"""

    def test_hidden_fields_form_sets_fields(self):
        """test_hidden_fields_form_sets_fields

        Returns:

        """
        form = HiddenFieldsForm(template_id="1", template_format="XSD")
        self.assertEqual(form.fields["template_id"].initial, "1")
        self.assertEqual(form.fields["template_format"].initial, "XSD")



"""Curate app user views
"""

import json
import logging
from builtins import any
from typing import Dict, List

from django.contrib import messages
from django.http import HttpResponseRedirect
from django.urls import reverse
from django.utils.decorators import method_decorator
from django.views import View
from django.conf import settings

import core_main_app.components.template.api as template_api
import core_main_app.components.template_version_manager.api as template_version_manager_api
from core_curate_app.components.curate_data_structure import (
    api as curate_data_structure_api,
)
from core_curate_app.permissions import rights
from core_curate_app.utils.parser import get_parser
from core_main_app.access_control.api import check_can_write
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.commons.constants import (
    DATA_FILE_CONTENT_TYPE_FOR_TEMPLATE_FORMAT,
    DATA_FILE_EXTENSION_FOR_TEMPLATE_FORMAT,
    TEMPLATE_FILE_EXTENSION_FOR_TEMPLATE_FORMAT,
    TEMPLATE_FILE_CONTENT_TYPE_FOR_TEMPLATE_FORMAT,
)
from core_main_app.commons.exceptions import (
    LockError,
    ModelError,
    DoesNotExist,
    XSDError,
)
from core_main_app.components.lock import api as lock_api
from core_main_app.components.template.models import Template
from core_main_app.utils import decorators
from core_main_app.utils.boolean import to_bool
from core_main_app.utils.file import get_file_http_response
from core_main_app.utils.labels import get_form_label
from core_main_app.utils.rendering import render
from core_main_app.utils.xml import format_content_xml
from core_parser_app.components.data_structure_element import (
    api as data_structure_element_api,
)
from core_parser_app.tools.parser.exceptions import ParserError
from core_parser_app.tools.parser.renderer.list import ListRenderer
from core_parser_app.tools.parser.renderer.xml import XmlRenderer

logger = logging.getLogger(__name__)

# TODO: Add a view for the registry. Ajax code need to be refactored


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
)
def index(request):
    """Page that allows to select a template to start curating.

    Args:
        request:

    Returns:

    """
    assets = {
        "js": [
            {
                "path": "core_curate_app/user/js/select_template.js",
                "is_raw": False,
            },
            {
                "path": "core_main_app/common/js/tooltip.js",
                "is_raw": False,
            },
            {
                "path": "core_curate_app/user/js/select_template.raw.js",
                "is_raw": True,
            },
        ],
        "css": [
            "core_curate_app/user/css/common.css",
            "core_curate_app/user/css/style.css",
        ],
    }

    global_active_template_list = (
        template_version_manager_api.get_active_global_version_manager(
            request=request
        )
    )
    user_active_template_list = (
        template_version_manager_api.get_active_version_manager_by_user_id(
            request=request
        )
    )

    context = {
        "templates_version_manager": global_active_template_list,
        "userTemplates": user_active_template_list,
    }

    # Set page title
    context.update({"page_title": "Curate"})

    return render(
        request,
        "core_curate_app/user/curate.html",
        assets=assets,
        context=context,
    )


# FIXME: allow reopening a form with unsaved changes (may be temporary until
#  curate workflow redesign)
class EnterDataView(View):
    """Page that allows to enter a data view."""

    def __init__(self):
        super().__init__()
        self.assets = {
            "js": [
                {
                    "path": "core_main_app/common/js/debounce.js",
                    "is_raw": False,
                },
                {
                    "path": "core_main_app/common/js/elementViewport.js",
                    "is_raw": False,
                },
                {
                    "path": "core_curate_app/user/js/enter_data.js",
                    "is_raw": False,
                },
                {
                    "path": "core_curate_app/user/js/enter_data.raw.js",
                    "is_raw": True,
                },
                {"path": "core_parser_app/js/modules.js", "is_raw": False},
                {"path": "core_parser_app/js/modules.raw.js", "is_raw": True},
                {
                    "path": "core_main_app/common/js/XMLTree.js",
                    "is_raw": False,
                },
                {"path": "core_parser_app/js/autosave.js", "is_raw": False},
                {"path": "core_parser_app/js/autosave.raw.js", "is_raw": True},
                {"path": "core_parser_app/js/buttons.js", "is_raw": False},
                {
                    "path": "core_curate_app/user/js/buttons.raw.js",
                    "is_raw": True,
                },
                {"path": "core_parser_app/js/choice.js", "is_raw": False},
                {
                    "path": "core_curate_app/user/js/choice.raw.js",
                    "is_raw": True,
                },
                {
                    "path": "core_main_app/common/js/data_detail.js",
                    "is_raw": False,
                },
                {
                    "path": "https://cdnjs.cloudflare.com/ajax/libs/json-editor/2.14.1/jsoneditor.js",
                    "integrity": "sha512-G93wD4PAiaCg3T4BeJGXNUdwJ4jr6WL0dKUqz0UwZs0jVad7FnJ2r+SlJ50k6+ILkYcQBqt4M5YoFcBWujIH0A==",
                    "is_external": True,
                    "is_raw": False,
                },
            ],
            "css": [
                "core_curate_app/user/css/common.css",
                "core_curate_app/user/css/xsd_form.css",
                "core_parser_app/css/use.css",
                "core_main_app/common/css/modals/download.css",
            ],
        }

        if settings.ENABLE_XML_ENTITIES_TOOLTIPS:
            self.assets["js"].append(
                {
                    "path": "core_curate_app/user/js/xml_entities_tooltip.js",
                    "is_raw": False,
                }
            )
            if settings.BOOTSTRAP_VERSION.startswith("4"):
                self.assets["js"].append(
                    {
                        "path": "core_curate_app/user/js/xml_entities_tooltip/popover.bs4.js",
                        "is_raw": False,
                    }
                )
            elif settings.BOOTSTRAP_VERSION.startswith("5"):
                self.assets["js"].append(
                    {
                        "path": "core_curate_app/user/js/xml_entities_tooltip/popover.bs5.js",
                        "is_raw": False,
                    }
                )

        self.modals = [
            "core_curate_app/user/data-entry/modals/cancel-changes.html",
            "core_curate_app/user/data-entry/modals/cancel-form.html",
            "core_curate_app/user/data-entry/modals/clear-fields.html",
            "core_main_app/common/modals/download-options.html",
            "core_curate_app/user/data-entry/modals/save-form.html",
            "core_curate_app/user/data-entry/modals/use-validation.html",
            "core_curate_app/user/data-entry/modals/validation-error.html",
            "core_curate_app/user/data-entry/modals/xml-valid.html",
            "core_curate_app/user/data-entry/modals/switch_to_text_editor.html",
        ]

    def build_context(
        self, request, curate_data_structure, reload_unsaved_changes
    ):
        """Build the context of the view

        Args:
            request:
            curate_data_structure:
            reload_unsaved_changes:

        Returns:

        """
        if curate_data_structure.template.format == Template.XSD:
            if reload_unsaved_changes:
                # get root element from the data structure
                root_element = (
                    curate_data_structure.data_structure_element_root
                )
            else:
                # if form string provided, use it to generate the form
                xml_string = curate_data_structure.form_string

                root_element = generate_root_element(
                    request, curate_data_structure, xml_string
                )

            # renders the form
            form = render_form(request, root_element)

        else:
            form = None

        return {
            "edit": True if curate_data_structure.data is not None else False,
            "form": form,
            "data_structure": curate_data_structure,
        }

    @method_decorator(
        decorators.permission_required(
            content_type=rights.CURATE_CONTENT_TYPE,
            permission=rights.CURATE_ACCESS,
        )
    )
    def get(
        self, request, curate_data_structure_id, reload_unsaved_changes=False
    ):
        """Load view to enter data.

        Args:
            request:
            curate_data_structure_id:
            reload_unsaved_changes:

        Returns:

        """
        curate_data_structure = None
        try:
            # Retrieve CurateDataStructure and lock the object for the current
            # user.
            curate_data_structure = _get_curate_data_structure_by_id(
                curate_data_structure_id, request
            )

            # Lock from database
            if curate_data_structure.data is not None:
                lock_api.set_lock_object(
                    curate_data_structure.data, request.user
                )

            # Build context
            context = self.build_context(
                request, curate_data_structure, reload_unsaved_changes
            )

            # additional style for the JSON form
            if curate_data_structure.template.format == Template.JSON:
                self.assets["css"].append(
                    "core_curate_app/user/css/json_form.css"
                )

            # Set page title
            context.update({"page_title": "Enter Data"})

            return render(
                request,
                "core_curate_app/user/data-entry/enter_data.html",
                assets=self.assets,
                context=context,
                modals=self.modals,
            )
        except (LockError, AccessControlError, ModelError, DoesNotExist) as ex:
            return render(
                request,
                "core_curate_app/user/errors.html",
                assets={},
                context={"errors": str(ex), "page_title": "Error"},
            )
        except ParserError as parser_exception:
            messages.add_message(
                self.request,
                messages.ERROR,
                str(parser_exception),
            )
            url = (
                reverse("core_curate_app_xml_text_editor_view")
                + f"?id={str(curate_data_structure_id)}"
            )
            return HttpResponseRedirect(url)
        except Exception as exception:
            try:
                # Unlock from database
                if (
                    curate_data_structure is not None
                    and curate_data_structure.data is not None
                ):
                    lock_api.remove_lock_on_object(
                        curate_data_structure.data, request.user
                    )
            except Exception as lock_exc:
                # CurateDataStructure not found, continue search
                logger.warning(
                    "'EnterDataView.get' threw an exception:%s", str(lock_exc)
                )

            return render(
                request,
                "core_curate_app/user/errors.html",
                assets={},
                context={"errors": str(exception), "page_title": "Error"},
            )


class ViewDataView(View):
    """
    Page that allows to view a data.

    Args:
        View:

    Returns:

    """

    def __init__(self):
        super().__init__()
        self.assets: Dict[str, List[any]] = {
            "js": [
                {
                    "path": "core_curate_app/user/js/view_data.js",
                    "is_raw": False,
                },
                {
                    "path": "core_curate_app/user/js/view_data.raw.js",
                    "is_raw": True,
                },
                {
                    "path": "core_main_app/common/js/XMLTree.js",
                    "is_raw": False,
                },
                {
                    "path": "core_main_app/common/js/data_detail.js",
                    "is_raw": False,
                },
            ],
            "css": [
                "core_main_app/common/css/XMLTree.css",
                "core_main_app/common/css/modals/download.css",
            ],
        }

        self.modals = [
            "core_curate_app/user/data-review/modals/save-error.html",
            "core_main_app/common/modals/download-options.html",
        ]

        if "core_file_preview_app" in settings.INSTALLED_APPS:
            self.assets["js"].extend(
                [
                    {
                        "path": "core_file_preview_app/user/js/file_preview.js",
                        "is_raw": False,
                    }
                ]
            )
            self.assets["css"].append(
                "core_file_preview_app/user/css/file_preview.css"
            )
            self.modals.append(
                "core_file_preview_app/user/file_preview_modal.html"
            )

    def build_context(self, request, curate_data_structure):
        """Build form string from CurateDataStructure

        Args:
            request:
            curate_data_structure:

        Returns:
        """
        if curate_data_structure.template.format == Template.XSD:
            content = render_xml(
                request, curate_data_structure.data_structure_element_root
            )
        else:
            content = curate_data_structure.form_string

        return {
            "edit": True if curate_data_structure.data is not None else False,
            "form_string": content,
            "data_structure": curate_data_structure,
        }

    @method_decorator(
        decorators.permission_required(
            content_type=rights.CURATE_CONTENT_TYPE,
            permission=rights.CURATE_ACCESS,
        )
    )
    def get(self, request, curate_data_structure_id):
        """Get data view

        Args:
            request:
            curate_data_structure_id:

        Returns:
        """

        try:
            curate_data_structure = _get_curate_data_structure_by_id(
                curate_data_structure_id, request
            )

            # Build Context
            context = self.build_context(request, curate_data_structure)
            # Set page title
            context.update({"page_title": "View Data"})

            return render(
                request,
                "core_curate_app/user/data-review/view_data.html",
                assets=self.assets,
                context=context,
                modals=self.modals,
            )
        except Exception as exception:
            return render(
                request,
                "core_curate_app/user/errors.html",
                assets={},
                context={"errors": str(exception), "page_title": "Error"},
            )


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
)
def download_current_document(request, curate_data_structure_id):
    """Make the current document available for download.

    Args:
        request:
        curate_data_structure_id:

    Returns:

    """
    # get curate data structure
    curate_data_structure = _get_curate_data_structure_by_id(
        curate_data_structure_id, request
    )
    # get pretty print bool
    prettify = request.GET.get("pretty_print", False)

    if curate_data_structure.template.format == Template.XSD:
        # generate xml string
        content = render_xml(
            request, curate_data_structure.data_structure_element_root
        )
        # prettify content
        if to_bool(prettify):
            content = format_content_xml(content)

    else:
        content = curate_data_structure.form_string
        # prettify content
        if to_bool(prettify):
            content = json.dumps(json.loads(content), indent=2)

    # build response with file
    return get_file_http_response(
        file_content=content,
        file_name=curate_data_structure.name,
        content_type=DATA_FILE_CONTENT_TYPE_FOR_TEMPLATE_FORMAT[
            curate_data_structure.template.format
        ],
        extension=DATA_FILE_EXTENSION_FOR_TEMPLATE_FORMAT[
            curate_data_structure.template.format
        ],
    )


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
)
def download_template(request, curate_data_structure_id):
    """Make the current template available for download.

    Args:
        request:
        curate_data_structure_id:

    Returns:

    """
    # get curate data structure
    curate_data_structure = _get_curate_data_structure_by_id(
        curate_data_structure_id, request
    )

    # get the template
    template = template_api.get_by_id(
        str(curate_data_structure.template.id), request=request
    )
    # get pretty print bool
    prettify = request.GET.get("pretty_print", False)

    # get the template content
    content = template.content
    if template.format == Template.XSD:
        # prettify content
        if to_bool(prettify):
            content = format_content_xml(content)
    else:
        if to_bool(prettify):
            content = json.dumps(json.loads(content), indent=2)

    # return the file
    return get_file_http_response(
        file_content=content,
        file_name=template.filename,
        content_type=TEMPLATE_FILE_CONTENT_TYPE_FOR_TEMPLATE_FORMAT[
            template.format
        ],
        extension=TEMPLATE_FILE_EXTENSION_FOR_TEMPLATE_FORMAT[template.format],
    )


def generate_form(
    xsd_string, xml_string=None, data_structure=None, request=None
):
    """Generate the form using the parser, returns the root element.

    Args:
        xsd_string:
        xml_string:
        data_structure:
        request:

    Returns:

    """
    # build parser
    parser = get_parser(request=request)
    # generate form
    root_element_id = parser.generate_form(
        xsd_string, xml_string, data_structure, request=request
    )
    # get the root element
    root_element = data_structure_element_api.get_by_id(
        root_element_id, request
    )

    return root_element


def render_form(request, root_element):
    """Render the form.

    Args:
        request:
        root_element:

    Returns:

    """
    # build a renderer
    renderer = ListRenderer(root_element, request)
    # render the form
    xsd_form = renderer.render()

    return xsd_form


def generate_root_element(request, curate_data_structure, xml_string):
    """Render the updated form.

    Args:
        request:
        curate_data_structure:
        xml_string:

    Returns:

    """
    # get xsd string from the template
    template = template_api.get_by_id(
        str(curate_data_structure.template.id), request=request
    )
    # check template format
    if template.format != Template.XSD:
        raise XSDError("Template format not supported.")

    # get the root element
    root_element = generate_form(
        template.content,
        xml_string,
        data_structure=curate_data_structure,
        request=request,
    )

    # save the root element in the data structure
    curate_data_structure_api.update_data_structure_root(
        curate_data_structure, root_element, request.user
    )

    return root_element


def render_xml(request, root_element):
    """Render the XML.

    Args:
        request:
        root_element:

    Returns:

    """
    # build XML renderer
    xml_renderer = XmlRenderer(root_element, request)

    # generate xml data
    xml_data = xml_renderer.render()

    return xml_data


def _get_curate_data_structure_by_id(curate_data_structure_id, request):
    """Get the curate data structure by its id.

    Args: curate_data_structure_id:
          request:
    Returns:
    """

    # get data structure
    curate_data_structure = curate_data_structure_api.get_by_id(
        curate_data_structure_id, request.user
    )

    # If not link to a data, only ownership check
    if curate_data_structure.data is None:
        # check ownership
        _check_owner(request, accessed_object=curate_data_structure)
    # Check based on the data
    else:
        # check can write data
        _check_can_write_data(request, accessed_object=curate_data_structure)

    return curate_data_structure


def _check_owner(request, accessed_object):
    """Check if the object can be accessed by the user.

    Args:
        request:
        accessed_object:

    Returns:

    """
    # Super user can access everything
    if not request.user.is_superuser:
        # If not the owner of the accessed object
        if str(request.user.id) != accessed_object.user:
            raise AccessControlError(
                "You are not the owner of the "
                + get_form_label()
                + " that you are trying to access"
            )


def _check_can_write_data(request, accessed_object):
    """Check if the object can be accessed by the user in write mode.

    Args:
        request:
        accessed_object:

    Returns:

    """
    try:
        # Super user can access everything
        if not request.user.is_superuser:
            check_can_write(accessed_object.data, request.user)
    except AccessControlError as ace:
        raise ace



"""AJAX views for the Curate app
"""

import json
import logging

from django.contrib import messages
from django.http.response import HttpResponseBadRequest, HttpResponse
from django.template import loader
from django.urls import reverse
from django.utils.html import escape
from lxml.etree import XMLSyntaxError

import core_curate_app.components.curate_data_structure.api as curate_data_structure_api
import core_curate_app.views.user.forms as users_forms
from core_curate_app.common import exceptions as exceptions
from core_curate_app.common.exceptions import CurateAjaxError
from core_curate_app.components.curate_data_structure.models import (
    CurateDataStructure,
)
from core_curate_app.permissions import rights as rights
from core_curate_app.utils.parser import get_parser
from core_curate_app.views.user import views as curate_user_views
from core_main_app.commons.exceptions import JSONError
from core_main_app.components.data import api as data_api
from core_main_app.components.data.models import Data
from core_main_app.components.lock import api as lock_api
from core_main_app.components.template import api as template_api
from core_main_app.components.template.models import Template
from core_main_app.utils import decorators
from core_main_app.utils import xml as main_xml_utils
from core_main_app.utils import json_utils as main_json_utils
from core_main_app.utils.json_utils import format_content_json
from core_main_app.utils.labels import get_data_label, get_form_label
from core_main_app.views.common import views as main_common_views
from core_parser_app.components.data_structure_element import (
    api as data_structure_element_api,
)
from core_parser_app.tools.parser.parser import remove_child_element
from core_parser_app.tools.parser.renderer.list import ListRenderer
from xml_utils.xsd_tree.xsd_tree import XSDTree

logger = logging.getLogger(__name__)


# FIXME: delete_branch not deleting all elements
# FIXME: generate element not testing max occurrences


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
    raise_exception=True,
)
def start_curate(request):
    """Load forms to start curating.

    Args:
        request:

    Returns:

    """
    try:
        if request.method == "POST":
            return _start_curate_post(request)

        return _start_curate_get(request)
    except CurateAjaxError as exception:
        return HttpResponseBadRequest(escape(str(exception)))
    except Exception as exception:
        return HttpResponseBadRequest(escape(str(exception)))


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
    raise_exception=True,
)
def generate_choice(request, curate_data_structure_id):
    """Generate a choice branch absent from the form.

    Args:
        request:
        curate_data_structure_id:

    Returns:

    """
    try:
        element_id = request.POST["id"]
        curate_data_structure = curate_data_structure_api.get_by_id(
            curate_data_structure_id, request.user
        )
        xsd_parser = get_parser(request=request)
        template = template_api.get_by_id(
            str(curate_data_structure.template.id), request=request
        )
        # check if xsd template
        if template.format != Template.XSD:
            return HttpResponseBadRequest("Template format not supported.")
        html_form = xsd_parser.generate_choice_absent(
            element_id, template.content, data_structure=curate_data_structure
        )
    except Exception as exception:
        return HttpResponseBadRequest(escape(str(exception)))

    return HttpResponse(html_form)


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
    raise_exception=True,
)
def generate_element(request, curate_data_structure_id):
    """Generate an element absent from the form.

    Args:
        request:
        curate_data_structure_id:

    Returns:

    """
    try:
        element_id = request.POST["id"]
        curate_data_structure = curate_data_structure_api.get_by_id(
            curate_data_structure_id, request.user
        )
        xsd_parser = get_parser(request=request)
        template = template_api.get_by_id(
            str(curate_data_structure.template.id), request=request
        )
        # check if xsd template
        if template.format != Template.XSD:
            return HttpResponseBadRequest("Template format not supported.")
        html_form = xsd_parser.generate_element_absent(
            element_id, template.content, data_structure=curate_data_structure
        )
    except Exception as exception:
        return HttpResponseBadRequest(escape(str(exception)))

    return HttpResponse(html_form)


# TODO: need to be reworked
@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
    raise_exception=True,
)
def remove_element(request):
    """Remove an element from the form.

    Args:
        request:

    Returns:

    """
    element_id = request.POST["id"]

    # Removing the element from the data structure
    data_structure_element_to_pull = data_structure_element_api.get_by_id(
        element_id, request
    )
    data_structure_element = data_structure_element_to_pull.parent

    # number of children after deletion
    children_number = data_structure_element.children.count() - 1

    data_structure_element = remove_child_element(
        data_structure_element, data_structure_element_to_pull, request
    )

    response = {"code": 0, "html": ""}

    if children_number > data_structure_element.options["min"]:
        return HttpResponse(json.dumps(response))

    # len(schema_element.children) == schema_element.options['min']
    if data_structure_element.options["min"] != 0:
        response["code"] = 1
    else:  # schema_element.options['min'] == 0
        renderer = ListRenderer(data_structure_element, request)
        html_form = renderer.render(True)

        response["code"] = 2
        response["html"] = html_form

    return HttpResponse(json.dumps(response))


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
    raise_exception=True,
)
def clear_fields(request):
    """Clear fields of the current form.

    Args:
        request:

    Returns:

    """
    try:
        # get curate data structure
        curate_data_structure_id = request.POST["id"]
        curate_data_structure = curate_data_structure_api.get_by_id(
            curate_data_structure_id, request.user
        )

        # generate form
        template = template_api.get_by_id(
            str(curate_data_structure.template.id), request=request
        )
        # check if xsd template
        if template.format != Template.XSD:
            return HttpResponseBadRequest("Template format not supported.")

        root_element = curate_user_views.generate_form(
            template.content,
            data_structure=curate_data_structure,
            request=request,
        )

        # save the root element in the data structure
        curate_data_structure_api.update_data_structure_root(
            curate_data_structure, root_element, request.user
        )

        # renders the form
        xsd_form = curate_user_views.render_form(request, root_element)

        return HttpResponse(
            json.dumps({"xsdForm": xsd_form}),
            content_type="application/javascript",
        )
    except Exception as exception:
        return HttpResponseBadRequest(escape(str(exception)))


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
    raise_exception=True,
)
def cancel_changes(request):
    """Cancel changes of the current form.

    Args:
        request:

    Returns:

    """
    try:
        # get curate data structure
        curate_data_structure_id = request.POST["id"]
        curate_data_structure = curate_data_structure_api.get_by_id(
            curate_data_structure_id, request.user
        )

        if curate_data_structure.data is not None:
            # data already saved, reload from data
            content = curate_data_structure.data.content
        elif curate_data_structure.form_string is not None:
            # form already saved, reload from saved form
            content = curate_data_structure.form_string
        else:
            # no saved data, load new form
            content = None

        if curate_data_structure.template.format == Template.XSD:
            root_element = curate_user_views.generate_root_element(
                request, curate_data_structure, content
            )

            # renders the form
            xsd_form = curate_user_views.render_form(request, root_element)
            result = {"xsdForm": xsd_form}
        elif curate_data_structure.template.format == Template.JSON:
            result = {"content": content}
        else:
            return HttpResponseBadRequest("Template format not supported.")

        return HttpResponse(
            json.dumps(result),
            content_type="application/javascript",
        )
    except Exception as exception:
        return HttpResponseBadRequest(escape(str(exception)))


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
    raise_exception=True,
)
def cancel_form(request):
    """Cancel current form.

    Args:
        request:

    Returns:

    """
    try:
        # get curate data structure
        curate_data_structure_id = request.POST["id"]
        curate_data_structure = curate_data_structure_api.get_by_id(
            curate_data_structure_id, request.user
        )
        # unlock from database
        if curate_data_structure.data is not None:
            lock_api.remove_lock_on_object(
                curate_data_structure.data, request.user
            )
        curate_data_structure_api.delete(curate_data_structure, request.user)

        # add success message
        messages.add_message(
            request,
            messages.SUCCESS,
            get_form_label().capitalize() + " deleted.",
        )

        return HttpResponse(json.dumps({}), content_type="application/json")
    except Exception as exception:
        error_message = (
            "An unexpected error has occurred while cancelling "
            f"the {get_form_label()}"
        )

        logger.error("%s: %s", error_message, str(exception))
        return HttpResponseBadRequest(
            json.dumps({"error": error_message}),
            content_type="application/json",
        )


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
    raise_exception=True,
)
def save_form(request):
    """Save the current form in data base. Converts it to XML format first.

    Args:
        request:

    Returns:

    """
    try:
        # get curate data structure
        curate_data_structure_id = request.POST["id"]
        curate_data_structure = curate_data_structure_api.get_by_id(
            curate_data_structure_id, request.user
        )

        if curate_data_structure.template.format == Template.XSD:
            form_string = request.POST.get("form_string", None)
            if form_string is None:
                form_string = curate_user_views.render_xml(
                    request,
                    curate_data_structure.data_structure_element_root,
                )
            else:
                # check if form string is well formed
                main_xml_utils.format_content_xml(form_string)

        elif curate_data_structure.template.format == Template.JSON:
            form_string = request.POST.get("form_string", "{}")
            # check if form string is well formed
            format_content_json(form_string)
        else:
            return HttpResponseBadRequest("Template format not supported.")

        # update curate data structure data
        curate_data_structure.form_string = form_string

        # save data structure
        curate_data_structure_api.upsert(curate_data_structure, request.user)

        return HttpResponse(
            json.dumps({"message": f"{get_form_label().capitalize()} saved."}),
            content_type="application/json",
        )
    except Exception as exception:
        error_message = (
            f"An error occurred while saving the {get_form_label()}"
        )
        logger.error("%s: %s", error_message, str(exception))
        return HttpResponseBadRequest(
            json.dumps({"error": error_message, "details": str(exception)}),
            content_type="application/json",
        )


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
    raise_exception=True,
)
def validate_form(request):
    """Validate data present in the form via XML validation.

    Args:
        request:

    Returns:

    """
    response_dict = {}
    try:
        # get curate data structure
        curate_data_structure_id = request.POST["id"]
        curate_data_structure = curate_data_structure_api.get_by_id(
            curate_data_structure_id, request.user
        )
        # get template
        template = template_api.get_by_id(
            str(curate_data_structure.template.id), request=request
        )
        # check if xsd template
        if template.format == Template.XSD:
            # generate the XML
            xml_data = curate_user_views.render_xml(
                request, curate_data_structure.data_structure_element_root
            )
            # build trees
            xsd_tree = XSDTree.build_tree(template.content)
            form_string = XSDTree.build_tree(xml_data)

            # validate XML document
            errors = main_xml_utils.validate_xml_data(
                xsd_tree, form_string, request=request
            )
            if errors is not None:
                response_dict["errors"] = errors
        elif template.format == Template.JSON:
            try:
                main_json_utils.validate_json_data(
                    (
                        request.POST["form_string"]
                        if "form_string" in request.POST
                        else curate_data_structure.form_string
                    ),
                    template.content,
                )
            except JSONError as json_error:
                response_dict["errors"] = [
                    escape(str(message)) for message in json_error.message_list
                ]
        else:
            return HttpResponseBadRequest("Template format not supported.")

    except XMLSyntaxError as xml_syntax_error:
        response_dict["errors"] = (
            "Your XML data is not well formatted. " + str(xml_syntax_error)
        )
    except Exception as exception:
        message = (
            str(exception).replace('"', "'")
            if str(exception) is not None
            else "The current document cannot be validated."
        )
        response_dict["errors"] = message

    return HttpResponse(
        json.dumps(response_dict), content_type="application/javascript"
    )


@decorators.permission_required(
    content_type=rights.CURATE_CONTENT_TYPE,
    permission=rights.CURATE_ACCESS,
    raise_exception=True,
)
def save_data(request):
    """Save data - delete curate data structure.

    Args:
        request:

    Returns:

    """
    try:
        # get curate data structure
        curate_data_structure_id = request.POST["id"]
        curate_data_structure = curate_data_structure_api.get_by_id(
            curate_data_structure_id, request.user
        )

        # unlock from database
        if curate_data_structure.data is not None:
            lock_api.remove_lock_on_object(
                curate_data_structure.data, request.user
            )

        if curate_data_structure.template.format == Template.XSD:
            # generate the XML
            form_string = curate_user_views.render_xml(
                request, curate_data_structure.data_structure_element_root
            )
        elif curate_data_structure.template.format == Template.JSON:
            form_string = curate_data_structure.form_string
        else:
            return HttpResponseBadRequest("Template format not supported.")

        if curate_data_structure.data is not None:
            # update existing data
            data = curate_data_structure.data
        else:
            # create new data
            data = Data()
            data.title = curate_data_structure.name
            template = template_api.get_by_id(
                str(curate_data_structure.template.id), request=request
            )
            data.template = template
            data.user_id = str(request.user.id)

        # set content
        data.content = form_string
        # save data
        data = data_api.upsert(data, request)

        curate_data_structure_api.delete(curate_data_structure, request.user)

        messages.add_message(
            request,
            messages.SUCCESS,
            get_data_label().capitalize() + " saved.",
        )
    except Exception as exception:
        return HttpResponseBadRequest(
            str(exception).replace('"', "'"),
            content_type="application/javascript",
        )

    return HttpResponse(
        json.dumps({"data_id": str(data.id)}),
        content_type="application/javascript",
    )


def _start_curate_post(request):
    """Start curate POST request.

    Args:
        request:

    Returns:

    """
    template_id = str(request.POST["template_id"])
    selected_option = request.POST["curate_form"]
    user_id = str(request.user.id)

    if selected_option == "new":
        new_form = users_forms.NewForm(request.POST)
        if not new_form.is_valid():
            raise CurateAjaxError(
                "An error occurred during the validation " + get_form_label()
            )
        name = new_form.data["document_name"]
        template = template_api.get_by_id(template_id, request=request)
        # check template format
        if (
            template.format != Template.XSD
            and template.format != Template.JSON
            and "text_editor" not in new_form.data
        ):
            return HttpResponseBadRequest("Template format not supported.")
        curate_data_structure = CurateDataStructure(
            user=user_id,
            template=template,
            name=name,
        )
        curate_data_structure_api.upsert(curate_data_structure, request.user)
        url = _get_reverse_url(new_form, curate_data_structure.id)
    elif selected_option == "upload":
        upload_form = users_forms.UploadForm(request.POST, request.FILES)
        if not upload_form.is_valid():
            raise CurateAjaxError(
                f"An error occurred during the file upload: {upload_form.errors.as_text()}"
            )
        # get template
        template = template_api.get_by_id(
            template_id,
            request=request,
        )
        file = request.FILES["file"]
        name = file.name
        # check template format
        if template.format == Template.XSD:
            content = main_common_views.read_xsd_file(file)
            well_formed = main_xml_utils.is_well_formed_xml(content)
        elif template.format == Template.JSON:
            content = file.read().decode("utf-8")
            well_formed = main_json_utils.is_well_formed_json(content)
        else:
            return HttpResponseBadRequest("Template format not supported.")
        if not well_formed:
            return HttpResponseBadRequest(
                "An error occurred during the file upload: the file is "
                "not well formed " + template.format
            )
        if (
            "direct_upload" in upload_form.data
            and upload_form.data["direct_upload"]
        ):
            # create data
            data = Data(
                title=name, template=template, user_id=str(request.user.id)
            )
            # set content
            data.content = content
            # save data
            data_api.upsert(data, request)
            messages.add_message(
                request,
                messages.SUCCESS,
                get_data_label().capitalize() + " saved.",
            )
            url = reverse("core_main_app_data_detail") + f"?id={str(data.id)}"
        else:
            curate_data_structure = CurateDataStructure(
                user=user_id,
                template=template_api.get_by_id(template_id, request=request),
                name=name,
                form_string=content,
            )
            curate_data_structure_api.upsert(
                curate_data_structure, request.user
            )
            url = _get_reverse_url(upload_form, curate_data_structure.id)
    else:
        open_form = users_forms.OpenForm(request.POST)
        curate_data_structure = curate_data_structure_api.get_by_id(
            open_form.data["forms"], request.user
        )
        url = _get_reverse_url(open_form, curate_data_structure.id)
    return HttpResponse(url)


def _get_reverse_url(form, curate_data_structure_id):
    """get reverse url.
    Args:
        form:
        curate_data_structure_id
    Returns:
        url:
    """
    if "text_editor" in form.data:
        if form.data["template_format"] == Template.JSON:
            url = (
                reverse("core_curate_app_json_text_editor_view")
                + f"?id={str(curate_data_structure_id)}"
            )
        else:
            url = (
                reverse("core_curate_app_xml_text_editor_view")
                + f"?id={str(curate_data_structure_id)}"
            )
    else:
        url = reverse(
            "core_curate_enter_data", args=(curate_data_structure_id,)
        )
    return url


def _start_curate_get(request):
    """Start curate GET request.

    Args:
        request:

    Returns:

    """
    try:
        context_params = dict()
        template_id = request.GET["template_id"]
        template = loader.get_template(
            "core_curate_app/user/curate_start.html"
        )

        open_form = users_forms.OpenForm(
            forms=curate_data_structure_api.get_all_by_user_id_and_template_id_with_no_data(
                request.user.id, template_id
            )
        )
        new_form = users_forms.NewForm()
        upload_form = users_forms.UploadForm()
        hidden_form = users_forms.HiddenFieldsForm(
            template_id=template_id,
            template_format=template_api.get_by_id(
                template_id, request=request
            ).format,
        )
        context_params["new_form"] = new_form
        context_params["open_form"] = open_form
        context_params["upload_form"] = upload_form
        context_params["hidden_form"] = hidden_form
        context_params["request"] = request
        context = {}

        context.update(context_params)
        return HttpResponse(
            json.dumps({"template": template.render(context)}),
            content_type="application/javascript",
        )
    except Exception as exception:
        logger.error(str(exception))
        raise exceptions.CurateAjaxError(
            "Error occurred during the " + get_form_label() + " display."
        )



""" Fixtures files for Data Structure
"""

from django.core.files.uploadedfile import SimpleUploadedFile

from core_curate_app.components.curate_data_structure.models import (
    CurateDataStructure,
)
from core_main_app.components.data.models import Data
from core_main_app.components.template.models import Template
from core_main_app.utils.integration_tests.fixture_interface import (
    FixtureInterface,
)
from core_parser_app.components.data_structure.models import (
    DataStructureElement,
)


class DataStructureFixtures(FixtureInterface):
    """Data structure fixtures"""

    data_structure_1 = None
    data_structure_2 = None
    data_structure_3 = None
    data_structure_root_element = None
    data_structure_json_1 = None
    data_structure_json_2 = None
    data = None
    data_json = None
    template_xsd = None
    template_json = None
    template_unsupported_format = None
    data_collection = None

    def insert_data(self):
        """Insert a set of Data.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_template()
        self.generate_data_collection()

    def generate_data_collection(self):
        """Generate a Data collection.

        Returns:

        """

        # NOTE: no xml_content to avoid using unsupported GridFS mock
        self.data = Data(
            template=self.template_xsd,
            user_id="1",
            dict_content=None,
            title="title",
        )
        self.data.save()

        self.data_json = Data(
            template=self.template_json,
            user_id="3",
            dict_content=None,
            title="title_json",
        )
        self.data_json.save()

        self.data_structure_root_element = DataStructureElement(
            user="3",
            tag="tag",
            value="value",
            options={},
        )
        self.data_structure_root_element.save()
        self.data_structure_1 = CurateDataStructure(
            user="1",
            template=self.template_xsd,
            name="data_structure_1",
            data=self.data,
        )
        self.data_structure_1.save()

        self.data_structure_2 = CurateDataStructure(
            user="1", template=self.template_xsd, name="data_structure_2"
        )
        self.data_structure_2.save()

        self.data_structure_3 = CurateDataStructure(
            user="2", template=self.template_xsd, name="data_structure_3"
        )
        self.data_structure_3.save()

        self.data_structure_json_1 = CurateDataStructure(
            user="3", template=self.template_json, name="data_structure_json_1"
        )
        self.data_structure_json_1.save()

        self.data_structure_json_2 = CurateDataStructure(
            user="3",
            template=self.template_json,
            name="data_structure_json_2",
            data=self.data_json,
        )
        self.data_structure_json_2.save()

        self.data_collection = [
            self.data_structure_1,
            self.data_structure_2,
            self.data_structure_3,
            self.data_structure_json_1,
            self.data_structure_json_2,
            self.data,
            self.data_json,
            self.data_structure_root_element,
        ]

    def generate_template(self):
        """Generate an unique Template.

        Returns:

        """
        self.template_xsd = Template()
        xsd = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:element name="tag"></xs:element></xs:schema>'
        )
        self.template_xsd.file = SimpleUploadedFile(
            "user1_template.xsd", xsd.encode("utf-8")
        )
        self.template_xsd.user = "1"
        self.template_xsd.content = xsd
        self.template_xsd.hash = ""
        self.template_xsd.filename = "filename1.xsd"
        self.template_xsd.save()

        self.template_json = Template()
        self.template_json.user = "3"
        self.template_json.file = SimpleUploadedFile(
            "user2_template.json", "{}".encode("utf-8")
        )
        self.template_json.content = "{}"
        self.template_json.hash = ""
        self.template_json.format = Template.JSON
        self.template_json.filename = "filename2.json"
        self.template_json.save()

        self.template_unsupported_format = Template()
        self.template_unsupported_format.user = "3"
        self.template_unsupported_format.file = SimpleUploadedFile(
            "template_unsupported_format", "{}".encode("utf-8")
        )
        self.template_unsupported_format.content = "{}"
        self.template_unsupported_format.hash = ""
        self.template_unsupported_format.format = "TEST"
        self.template_unsupported_format.filename = (
            "template_unsupported_format"
        )
        self.template_unsupported_format.save()


class DataStructureFixtures2(FixtureInterface):
    """Data structure fixtures"""

    data_structure_1 = None
    data_structure_2 = None
    data_structure_3 = None
    data = None
    data_without_draft = None
    data_multiple_drafts = None
    template = None
    data_collection = None

    def insert_data(self):
        """Insert a set of Data.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_template()
        self.generate_data_collection()

    def generate_data_collection(self):
        """Generate a Data collection.

        Returns:

        """

        # NOTE: no xml_content to avoid using unsupported GridFS mock
        self.data = Data(
            template=self.template,
            user_id="1",
            dict_content=None,
            title="title",
        )
        self.data.save()

        self.data_without_draft = Data(
            template=self.template,
            user_id="1",
            dict_content=None,
            title="title",
        )
        self.data_without_draft.save()

        self.data_multiple_drafts = Data(
            template=self.template,
            user_id="1",
            dict_content=None,
            title="title",
        )
        self.data_multiple_drafts.save()

        self.data_structure_1 = CurateDataStructure(
            user="1",
            template=self.template,
            name="data_structure_1",
            data=self.data,
        )
        self.data_structure_1.save()

        self.data_structure_2 = CurateDataStructure(
            user="1",
            template=self.template,
            name="data_structure_2",
            data=self.data_multiple_drafts,
        )
        self.data_structure_2.save()

        self.data_structure_3 = CurateDataStructure(
            user="2",
            template=self.template,
            name="data_structure_3",
            data=self.data_multiple_drafts,
        )
        self.data_structure_3.save()

        self.data_collection = [
            self.data_structure_1,
            self.data_structure_2,
            self.data_structure_3,
            self.data,
            self.data_without_draft,
            self.data_multiple_drafts,
        ]

    def generate_template(self):
        """Generate an unique Template.

        Returns:

        """
        self.template = Template()
        xsd = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:element name="tag"></xs:element></xs:schema>'
        )
        self.template.file = SimpleUploadedFile(
            "user1_template.xsd", xsd.encode("utf-8")
        )
        self.template.user = "1"
        self.template.content = xsd
        self.template.hash = ""
        self.template.filename = "filename.xsd"
        self.template.save()


class DataStructureRootElementsFixtures(FixtureInterface):
    """Data structure fixtures"""

    data_structure_1 = None
    data_structure_2 = None
    data_structure_root_element = None
    template = None
    data_collection = None

    def insert_data(self):
        """Insert a set of Data.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_template()
        self.generate_data_collection()

    def generate_data_collection(self):
        """Generate a Data collection.

        Returns:

        """

        self.data_structure_root_element = DataStructureElement(
            user="3",
            tag="tag",
            value="value",
            options={},
        )
        self.data_structure_root_element.save()
        self.data_structure_1 = CurateDataStructure(
            user="1", template=self.template, name="data_structure_1"
        )
        self.data_structure_1.save()

        self.data_structure_2 = CurateDataStructure(
            user="1", template=self.template, name="data_structure_2"
        )
        self.data_structure_2.save()

        self.data_collection = [
            self.data_structure_1,
            self.data_structure_2,
            self.data_structure_root_element,
        ]

    def generate_template(self):
        """Generate an unique Template.

        Returns:

        """
        self.template = Template()
        xsd = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:element name="tag"></xs:element></xs:schema>'
        )
        self.template.file = SimpleUploadedFile(
            "user1_template.xsd", xsd.encode("utf-8")
        )

        self.template.user = "1"
        self.template.content = xsd
        self.template.hash = ""
        self.template.filename = "demo_diff.xsd"
        self.template.save()



""" Url router for the curate application
"""

from django.conf.urls import include
from django.urls import re_path


urlpatterns = [
    re_path(r"^", include("core_main_app.urls")),
    re_path(r"^", include("core_curate_app.urls")),
]



""" Unit tests for views from `views.user.views`.
"""

from unittest.case import TestCase
from unittest.mock import patch, MagicMock

from django.http import (
    HttpResponseRedirect,
)
from django.test import RequestFactory, override_settings
from django.urls import reverse

from core_curate_app.views.user import views as curate_user_views
from core_curate_app.views.user.views import (
    EnterDataView,
    ViewDataView,
    generate_root_element,
)
from core_main_app.commons.exceptions import XSDError
from core_main_app.components.template.models import Template
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.xml import format_content_xml
from core_parser_app.tools.parser.exceptions import ParserError


class TestEnterDataView(TestCase):
    """Test EnterDataView"""

    def setUp(self):
        """setUp
        Returns:
        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.user1.has_perm = MagicMock()
        self.user1.has_perm.return_value = True

    @patch("core_curate_app.views.user.views.EnterDataView.build_context")
    @patch("core_curate_app.views.user.views._get_curate_data_structure_by_id")
    def test_enter_data_view_get_returns_error_if_parser_error_occurs(
        self, mock_ds_get_by_id, mock_build_context
    ):
        """test_enter_data_view_get_returns_error_if_parser_error_occurs"""
        # Arrange
        mock_build_context.side_effect = ParserError("error")
        mock_ds = MagicMock()
        mock_ds.form_string = "test"
        mock_ds.data = None
        mock_ds_get_by_id.return_value = mock_ds
        request = self.factory.get("core_curate_enter_data")
        request._messages = MagicMock()
        request.user = self.user1
        # Act
        response = EnterDataView.as_view()(request, curate_data_structure_id=1)
        # Assert
        self.assertTrue(isinstance(response, HttpResponseRedirect))
        self.assertTrue(
            response.url.startswith(
                reverse("core_curate_app_xml_text_editor_view")
            )
        )

    @override_settings(ENABLE_XML_ENTITIES_TOOLTIPS=True)
    def test_enable_xml_entities_tootip_true_includes_base_js_asset(self):
        """test_enable_xml_entities_tootip_true_includes_base_js_asset"""
        result = EnterDataView()

        self.assertIn(
            {
                "path": "core_curate_app/user/js/xml_entities_tooltip.js",
                "is_raw": False,
            },
            result.assets["js"],
        )

    @override_settings(ENABLE_XML_ENTITIES_TOOLTIPS=False)
    def test_enable_xml_entities_tootip_false_includes_no_extra_js(self):
        """test_enable_xml_entities_tootip_false_includes_no_extra_js"""
        result = EnterDataView()

        self.assertNotIn(
            {
                "path": "core_curate_app/user/js/xml_entities_tooltip.js",
                "is_raw": False,
            },
            result.assets["js"],
        )

    @override_settings(BOOTSTRAP_VERSION="4.6.2")
    def test_bootstrap_4_6_2_includes_correct_tooltip_js_asset(self):
        """test_bootstrap_4_6_2_includes_correct_tooltip_js_asset"""
        result = EnterDataView()

        self.assertIn(
            {
                "path": "core_curate_app/user/js/xml_entities_tooltip/popover.bs4.js",
                "is_raw": False,
            },
            result.assets["js"],
        )

    @override_settings(BOOTSTRAP_VERSION="5.1.3")
    def test_bootstrap_5_1_3_includes_correct_tooltip_js_asset(self):
        """test_bootstrap_5_1_3_includes_correct_tooltip_js_asset"""
        result = EnterDataView()

        self.assertIn(
            {
                "path": "core_curate_app/user/js/xml_entities_tooltip/popover.bs5.js",
                "is_raw": False,
            },
            result.assets["js"],
        )

    @patch.object(curate_user_views, "render_form")
    @patch("core_curate_app.views.user.views.generate_root_element")
    @patch("core_curate_app.views.user.views._get_curate_data_structure_by_id")
    def test_enter_data_view_build_context_with_xsd_template_returns_http_200(
        self, mock_ds_get_by_id, mock_generate_root_element, mock_render_form
    ):
        """test_enter_data_view_build_context_with_xsd_template_returns_http_200"""
        # Arrange
        mock_ds = MagicMock()
        mock_ds.form_string = "<root></root>"
        mock_ds.data = None
        mock_ds.template = Template(format=Template.XSD)
        mock_ds_get_by_id.return_value = mock_ds
        request = self.factory.get("core_curate_enter_data")
        request._messages = MagicMock()
        request.user = self.user1
        mock_generate_root_element.return_value = None
        mock_render_form.return_value = None
        # Act
        response = EnterDataView.as_view()(request, curate_data_structure_id=1)
        # Assert
        self.assertEqual(response.status_code, 200)

    @patch.object(curate_user_views, "render_form")
    @patch("core_curate_app.views.user.views.generate_root_element")
    @patch("core_curate_app.views.user.views._get_curate_data_structure_by_id")
    def test_enter_data_view_build_context_with_unsaved_changes_returns_http_200(
        self, mock_ds_get_by_id, mock_generate_root_element, mock_render_form
    ):
        """test_enter_data_view_build_context_with_unsaved_changes_returns_http_200"""
        # Arrange
        mock_ds = MagicMock()
        mock_ds.form_string = "<root></root>"
        mock_ds.data = None
        mock_ds.template = Template(format=Template.XSD)
        mock_ds_get_by_id.return_value = mock_ds
        request = self.factory.get("core_curate_enter_data")
        request._messages = MagicMock()
        request.user = self.user1
        mock_generate_root_element.return_value = None
        mock_render_form.return_value = None
        # Act
        response = EnterDataView.as_view()(
            request, curate_data_structure_id=1, reload_unsaved_changes=True
        )
        # Assert
        self.assertEqual(response.status_code, 200)

    @patch.object(curate_user_views, "render")
    @patch.object(curate_user_views, "render_form")
    @patch("core_curate_app.views.user.views.generate_root_element")
    @patch("core_curate_app.views.user.views._get_curate_data_structure_by_id")
    def test_enter_data_xml_includes_no_extra_css(
        self,
        mock_ds_get_by_id,
        mock_generate_root_element,
        mock_render_form,
        mock_render,
    ):
        """test_enter_data_xml_includes_no_extra_css"""
        mock_ds = MagicMock()
        mock_ds.form_string = "test"
        mock_ds.template = Template(format=Template.XSD)
        mock_ds.data = None
        mock_ds_get_by_id.return_value = mock_ds
        request = self.factory.get("core_curate_enter_data")
        request._messages = MagicMock()
        request.user = self.user1
        mock_generate_root_element.return_value = None
        mock_render_form.return_value = None
        expected_context = {
            "edit": False,
            "form": None,
            "data_structure": mock_ds,
            "page_title": "Enter Data",
        }
        # Act
        EnterDataView.as_view()(request, curate_data_structure_id=2)

        # Assert
        mock_render.assert_called_with(
            request,
            "core_curate_app/user/data-entry/enter_data.html",
            **{
                "assets": {
                    "js": [
                        {
                            "path": "core_main_app/common/js/debounce.js",
                            "is_raw": False,
                        },
                        {
                            "path": "core_main_app/common/js/elementViewport.js",
                            "is_raw": False,
                        },
                        {
                            "path": "core_curate_app/user/js/enter_data.js",
                            "is_raw": False,
                        },
                        {
                            "path": "core_curate_app/user/js/enter_data.raw.js",
                            "is_raw": True,
                        },
                        {
                            "path": "core_parser_app/js/modules.js",
                            "is_raw": False,
                        },
                        {
                            "path": "core_parser_app/js/modules.raw.js",
                            "is_raw": True,
                        },
                        {
                            "path": "core_main_app/common/js/XMLTree.js",
                            "is_raw": False,
                        },
                        {
                            "path": "core_parser_app/js/autosave.js",
                            "is_raw": False,
                        },
                        {
                            "path": "core_parser_app/js/autosave.raw.js",
                            "is_raw": True,
                        },
                        {
                            "path": "core_parser_app/js/buttons.js",
                            "is_raw": False,
                        },
                        {
                            "path": "core_curate_app/user/js/buttons.raw.js",
                            "is_raw": True,
                        },
                        {
                            "path": "core_parser_app/js/choice.js",
                            "is_raw": False,
                        },
                        {
                            "path": "core_curate_app/user/js/choice.raw.js",
                            "is_raw": True,
                        },
                        {
                            "path": "core_main_app/common/js/data_detail.js",
                            "is_raw": False,
                        },
                        {
                            "path": "https://cdnjs.cloudflare.com/ajax/libs/json-editor/2.14.1/jsoneditor.js",
                            "integrity": "sha512-G93wD4PAiaCg3T4BeJGXNUdwJ4jr6WL0dKUqz0UwZs0jVad7FnJ2r+SlJ50k6+ILkYcQBqt4M5YoFcBWujIH0A==",
                            "is_external": True,
                            "is_raw": False,
                        },
                        {
                            "path": "core_curate_app/user/js/xml_entities_tooltip.js",
                            "is_raw": False,
                        },
                        {
                            "path": "core_curate_app/user/js/xml_entities_tooltip/popover.bs4.js",
                            "is_raw": False,
                        },
                    ],
                    "css": [
                        "core_curate_app/user/css/common.css",
                        "core_curate_app/user/css/xsd_form.css",
                        "core_parser_app/css/use.css",
                        "core_main_app/common/css/modals/download.css",
                    ],
                },
                "context": expected_context,
                "modals": [
                    "core_curate_app/user/data-entry/modals/cancel-changes.html",
                    "core_curate_app/user/data-entry/modals/cancel-form.html",
                    "core_curate_app/user/data-entry/modals/clear-fields.html",
                    "core_main_app/common/modals/download-options.html",
                    "core_curate_app/user/data-entry/modals/save-form.html",
                    "core_curate_app/user/data-entry/modals/use-validation.html",
                    "core_curate_app/user/data-entry/modals/validation-error.html",
                    "core_curate_app/user/data-entry/modals/xml-valid.html",
                    "core_curate_app/user/data-entry/modals/switch_to_text_editor.html",
                ],
            },
        )

    @patch("core_curate_app.views.user.views._get_curate_data_structure_by_id")
    def test_enter_data_view_build_context_with_json_template_returns_http_200(
        self, mock_ds_get_by_id
    ):
        """test_enter_data_view_build_context_with_json_template_returns_http_200"""
        # Arrange
        mock_ds = MagicMock()
        mock_ds.form_string = "test"
        mock_ds.template = Template(format=Template.JSON)
        mock_ds.data = None
        mock_ds_get_by_id.return_value = mock_ds
        request = self.factory.get("core_curate_enter_data")
        request._messages = MagicMock()
        request.user = self.user1
        # Act
        response = EnterDataView.as_view()(request, curate_data_structure_id=1)
        # Assert
        self.assertEqual(response.status_code, 200)

    @patch.object(curate_user_views, "render")
    @patch("core_curate_app.views.user.views._get_curate_data_structure_by_id")
    def test_enter_data_json_includes_extra_css(
        self, mock_ds_get_by_id, mock_render
    ):
        """test_enter_data_json_includes_extra_css"""
        mock_ds = MagicMock()
        mock_ds.form_string = "test"
        mock_ds.template = Template(format=Template.JSON)
        mock_ds.data = None
        mock_ds_get_by_id.return_value = mock_ds
        request = self.factory.get("core_curate_enter_data")
        request._messages = MagicMock()
        request.user = self.user1
        expected_context = {
            "edit": False,
            "form": None,
            "data_structure": mock_ds,
            "page_title": "Enter Data",
        }
        # Act
        EnterDataView.as_view()(request, curate_data_structure_id=1)

        # Assert
        mock_render.assert_called_with(
            request,
            "core_curate_app/user/data-entry/enter_data.html",
            **{
                "assets": {
                    "js": [
                        {
                            "path": "core_main_app/common/js/debounce.js",
                            "is_raw": False,
                        },
                        {
                            "path": "core_main_app/common/js/elementViewport.js",
                            "is_raw": False,
                        },
                        {
                            "path": "core_curate_app/user/js/enter_data.js",
                            "is_raw": False,
                        },
                        {
                            "path": "core_curate_app/user/js/enter_data.raw.js",
                            "is_raw": True,
                        },
                        {
                            "path": "core_parser_app/js/modules.js",
                            "is_raw": False,
                        },
                        {
                            "path": "core_parser_app/js/modules.raw.js",
                            "is_raw": True,
                        },
                        {
                            "path": "core_main_app/common/js/XMLTree.js",
                            "is_raw": False,
                        },
                        {
                            "path": "core_parser_app/js/autosave.js",
                            "is_raw": False,
                        },
                        {
                            "path": "core_parser_app/js/autosave.raw.js",
                            "is_raw": True,
                        },
                        {
                            "path": "core_parser_app/js/buttons.js",
                            "is_raw": False,
                        },
                        {
                            "path": "core_curate_app/user/js/buttons.raw.js",
                            "is_raw": True,
                        },
                        {
                            "path": "core_parser_app/js/choice.js",
                            "is_raw": False,
                        },
                        {
                            "path": "core_curate_app/user/js/choice.raw.js",
                            "is_raw": True,
                        },
                        {
                            "path": "core_main_app/common/js/data_detail.js",
                            "is_raw": False,
                        },
                        {
                            "path": "https://cdnjs.cloudflare.com/ajax/libs/json-editor/2.14.1/jsoneditor.js",
                            "integrity": "sha512-G93wD4PAiaCg3T4BeJGXNUdwJ4jr6WL0dKUqz0UwZs0jVad7FnJ2r+SlJ50k6+ILkYcQBqt4M5YoFcBWujIH0A==",
                            "is_external": True,
                            "is_raw": False,
                        },
                        {
                            "path": "core_curate_app/user/js/xml_entities_tooltip.js",
                            "is_raw": False,
                        },
                        {
                            "path": "core_curate_app/user/js/xml_entities_tooltip/popover.bs4.js",
                            "is_raw": False,
                        },
                    ],
                    "css": [
                        "core_curate_app/user/css/common.css",
                        "core_curate_app/user/css/xsd_form.css",
                        "core_parser_app/css/use.css",
                        "core_main_app/common/css/modals/download.css",
                        "core_curate_app/user/css/json_form.css",
                    ],
                },
                "context": expected_context,
                "modals": [
                    "core_curate_app/user/data-entry/modals/cancel-changes.html",
                    "core_curate_app/user/data-entry/modals/cancel-form.html",
                    "core_curate_app/user/data-entry/modals/clear-fields.html",
                    "core_main_app/common/modals/download-options.html",
                    "core_curate_app/user/data-entry/modals/save-form.html",
                    "core_curate_app/user/data-entry/modals/use-validation.html",
                    "core_curate_app/user/data-entry/modals/validation-error.html",
                    "core_curate_app/user/data-entry/modals/xml-valid.html",
                    "core_curate_app/user/data-entry/modals/switch_to_text_editor.html",
                ],
            },
        )


class TestViewDataViewInit(TestCase):
    """Unit tests for `ViewDataView.__init__` method."""

    def setUp(self):
        """setUp
        Returns:
        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.user1.has_perm = MagicMock()
        self.user1.has_perm.return_value = True

    @override_settings(INSTALLED_APPS=["core_file_preview_app"])
    def test_file_preview_app_installed_adds_correct_js_asset(self):
        """test_file_preview_app_installed_adds_correct_js_asset"""
        result = ViewDataView()

        self.assertIn(
            {
                "path": "core_file_preview_app/user/js/file_preview.js",
                "is_raw": False,
            },
            result.assets["js"],
        )

    @override_settings(INSTALLED_APPS=["core_file_preview_app"])
    def test_file_preview_app_installed_adds_correct_css_asset(self):
        """test_file_preview_app_installed_adds_correct_css_asset"""
        result = ViewDataView()

        self.assertIn(
            "core_file_preview_app/user/css/file_preview.css",
            result.assets["css"],
        )

    @override_settings(INSTALLED_APPS=["core_file_preview_app"])
    def test_file_preview_app_installed_adds_correct_modal_asset(self):
        """test_file_preview_app_installed_adds_correct_modal_asset"""
        result = ViewDataView()

        self.assertIn(
            "core_file_preview_app/user/file_preview_modal.html", result.modals
        )

    @override_settings(INSTALLED_APPS=[])
    def test_file_preview_app_not_installed_adds_no_js_asset(self):
        """test_file_preview_app_not_installed_adds_no_js_asset"""
        result = ViewDataView()

        self.assertNotIn(
            {
                "path": "core_file_preview_app/user/js/file_preview.js",
                "is_raw": False,
            },
            result.assets["js"],
        )

    @override_settings(INSTALLED_APPS=[])
    def test_file_preview_app_not_installed_adds_no_css_asset(self):
        """test_file_preview_app_not_installed_adds_no_css_asset"""
        result = ViewDataView()

        self.assertNotIn(
            "core_file_preview_app/user/css/file_preview.css",
            result.assets["css"],
        )

    @override_settings(INSTALLED_APPS=[])
    def test_file_preview_app_not_installed_adds_no_modal_asset(self):
        """test_file_preview_app_not_installed_adds_no_modal_asset"""
        result = ViewDataView()

        self.assertNotIn(
            "core_file_preview_app/user/file_preview_modal.html", result.modals
        )

    @patch.object(curate_user_views, "render_xml")
    @patch("core_curate_app.views.user.views._get_curate_data_structure_by_id")
    def test_view_data_view_build_context_with_xsd_template_returns_http_200(
        self, mock_ds_get_by_id, mock_render_xml
    ):
        """test_enter_data_view_build_context_with_xsd_template_returns_http_200"""
        # Arrange
        mock_ds = MagicMock()
        mock_ds.form_string = "<root></root>"
        mock_ds.template = Template(format=Template.XSD)
        mock_ds_get_by_id.return_value = mock_ds
        request = self.factory.get("core_curate_view_data")
        request.user = self.user1
        mock_render_xml.return_value = None
        # Act
        response = ViewDataView.as_view()(request, curate_data_structure_id=1)
        # Assert
        self.assertEqual(response.status_code, 200)

    @patch("core_curate_app.views.user.views._get_curate_data_structure_by_id")
    def test_view_data_view_build_context_with_json_template_returns_http_200(
        self, mock_ds_get_by_id
    ):
        """test_view_data_view_build_context_with_json_template_returns_http_200"""
        # Arrange
        mock_ds = MagicMock()
        mock_ds.form_string = "{}"
        mock_ds.template = Template(format=Template.JSON)
        mock_ds_get_by_id.return_value = mock_ds
        request = self.factory.get("core_curate_view_data")
        request.user = self.user1
        # Act
        response = ViewDataView.as_view()(request, curate_data_structure_id=1)
        # Assert
        self.assertEqual(response.status_code, 200)


class TestGenerateRootElementView(TestCase):
    """Unit tests for `generate_root_element` method."""

    def setUp(self):
        """setUp"""
        factory = RequestFactory()
        user1 = create_mock_user(user_id="1", has_perm=True)

        self.request = factory.get("core_curate_cancel_changes")
        self.request.POST = {"id": 1}
        self.request.user = user1

    @patch("core_main_app.components.template.api.get_by_id")
    def test_generate_element_root_raises_xsd_error_if_not_xsd_template(
        self, mock_template_get_by_id
    ):
        """test_generate_element_root_raises_xsd_error_if_not_xsd_template"""
        mock_template = _get_json_template()
        mock_curate_data_structure = MagicMock(template=mock_template)
        mock_template_get_by_id.return_value = mock_template

        with self.assertRaises(XSDError):
            generate_root_element(self.request, mock_curate_data_structure, "")
        self.assertTrue(mock_template_get_by_id.called)


class TestDownloadCurrentDocumentView(TestCase):
    """Test Download Current Document View"""

    def setUp(self):
        """setUp"""
        factory = RequestFactory()
        user1 = create_mock_user(user_id="1", has_perm=True)

        self.request = factory.get("core_curate_download_document")
        self.request.user = user1

    @patch.object(curate_user_views, "render_xml")
    @patch.object(curate_user_views, "curate_data_structure_api")
    def test_download_xml_document_returns_file_without_formatting(
        self,
        mock_curate_data_structure_api,
        mock_render_xml,
    ):
        """test_download_xml_document_returns_file_without_formatting"""
        mock_data = MagicMock(
            template=Template(format=Template.XSD), data=None, user="1"
        )
        mock_curate_data_structure_api.get_by_id.return_value = mock_data

        mock_render_xml.return_value = "<root></root>"

        response = curate_user_views.download_current_document(self.request, 1)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b"<root></root>")

    @patch.object(curate_user_views, "render_xml")
    @patch.object(curate_user_views, "curate_data_structure_api")
    def test_download_xml_document_returns_file_with_formatting_option(
        self,
        mock_curate_data_structure_api,
        mock_render_xml,
    ):
        """test_download_xml_document_returns_file_with_formatting_option"""
        mock_data = MagicMock(
            template=Template(format=Template.XSD), data=None, user="1"
        )
        mock_curate_data_structure_api.get_by_id.return_value = mock_data

        mock_render_xml.return_value = "<root></root>"

        self.request.GET = {"pretty_print": True}
        response = curate_user_views.download_current_document(self.request, 1)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b"<root/>\n")

    @patch.object(curate_user_views, "curate_data_structure_api")
    def test_download_json_document_returns_file_without_formatting(
        self,
        mock_curate_data_structure_api,
    ):
        """test_download_xml_document_returns_file"""
        mock_data = MagicMock(
            template=Template(format=Template.JSON),
            data=None,
            user="1",
            form_string="",
        )
        mock_curate_data_structure_api.get_by_id.return_value = mock_data

        response = curate_user_views.download_current_document(self.request, 1)

        self.assertEqual(response.status_code, 200)

    @patch.object(curate_user_views, "curate_data_structure_api")
    def test_download_json_document_returns_formatted_file(
        self,
        mock_curate_data_structure_api,
    ):
        """test_download_xml_document_returns_file"""
        mock_data = MagicMock(
            template=Template(format=Template.JSON),
            data=None,
            user="1",
            form_string="{}",
        )
        mock_curate_data_structure_api.get_by_id.return_value = mock_data

        self.request.GET = {"pretty_print": True}
        response = curate_user_views.download_current_document(self.request, 1)

        self.assertEqual(response.status_code, 200)


class TestDownloadTemplateView(TestCase):
    """Test Download Template View"""

    def setUp(self):
        """setUp"""
        factory = RequestFactory()
        user1 = create_mock_user(user_id="1", has_perm=True)

        self.request = factory.get("core_curate_download_template")
        self.request.user = user1

    @patch("core_main_app.components.template.api.get_by_id")
    @patch.object(curate_user_views, "curate_data_structure_api")
    def test_download_xsd_template_returns_file_without_formatting(
        self,
        mock_curate_data_structure_api,
        mock_template_get_by_id,
    ):
        """test_download_xsd_template_returns_file_without_formatting"""

        content = (
            '<?xml version="1.0" encoding="UTF-8"?>'
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            "</xs:schema>"
        )

        mock_template = Template(format=Template.XSD, content=content)
        mock_data = MagicMock(template=mock_template, data=None, user="1")

        mock_curate_data_structure_api.get_by_id.return_value = mock_data

        mock_template_get_by_id.return_value = mock_template

        response = curate_user_views.download_template(self.request, 1)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode("utf-8"), content)

    @patch("core_main_app.components.template.api.get_by_id")
    @patch.object(curate_user_views, "curate_data_structure_api")
    def test_download_xsd_template_returns_formatted_file(
        self,
        mock_curate_data_structure_api,
        mock_template_get_by_id,
    ):
        """test_download_xsd_template_returns_formatted_file"""
        content = (
            '<?xml version="1.0" encoding="UTF-8"?>'
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            "</xs:schema>"
        )

        mock_template = Template(format=Template.XSD, content=content)
        mock_data = MagicMock(template=mock_template, data=None, user="1")

        mock_curate_data_structure_api.get_by_id.return_value = mock_data

        mock_template_get_by_id.return_value = mock_template

        self.request.GET = {"pretty_print": True}
        response = curate_user_views.download_template(self.request, 1)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.content.decode("utf-8"), format_content_xml(content)
        )

    @patch("core_main_app.components.template.api.get_by_id")
    @patch.object(curate_user_views, "curate_data_structure_api")
    def test_download_json_template_returns_file_without_formatting(
        self, mock_curate_data_structure_api, mock_template_get_by_id
    ):
        """test_download_json_template_returns_file_without_formatting"""
        mock_template = Template(format=Template.JSON, content="{}")
        mock_data = MagicMock(template=mock_template, data=None, user="1")
        mock_curate_data_structure_api.get_by_id.return_value = mock_data
        mock_template_get_by_id.return_value = mock_template

        response = curate_user_views.download_template(self.request, 1)

        self.assertEqual(response.status_code, 200)

    @patch("core_main_app.components.template.api.get_by_id")
    @patch.object(curate_user_views, "curate_data_structure_api")
    def test_download_json_document_returns_formatted_file(
        self,
        mock_curate_data_structure_api,
        mock_template_get_by_id,
    ):
        """test_download_json_document_returns_formatted_file"""
        mock_template = Template(format=Template.JSON, content="{}")
        mock_data = MagicMock(template=mock_template, data=None, user="1")
        mock_curate_data_structure_api.get_by_id.return_value = mock_data
        mock_template_get_by_id.return_value = mock_template

        self.request.GET = {"pretty_print": True}
        response = curate_user_views.download_template(self.request, 1)

        self.assertEqual(response.status_code, 200)


def _get_json_template():
    """Get JSON template

    Returns:

    """
    template = Template()
    template.format = Template.JSON
    template.id_field = 1
    template.content = "{}"
    return template
